@@ -1,7 +1,7 @@
 /* mpfr_eint, mpfr_eint1 -- the exponential integral
 
-Copyright 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2005-2016 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -206,7 +206,7 @@ mpfr_eint (mpfr_ptr y, mpfr_srcptr x, mpfr_rnd_t rnd)
 
   if (MPFR_UNLIKELY (MPFR_IS_SINGULAR (x)))
     {
-      /* exp(NaN) = exp(-Inf) = NaN */
+      /* eint(NaN) = eint(-Inf) = NaN */
       if (MPFR_IS_NAN (x) || (MPFR_IS_INF (x) && MPFR_IS_NEG(x)))
         {
           MPFR_SET_NAN (y);
@@ -308,7 +308,7 @@ mpfr_eint (mpfr_ptr y, mpfr_srcptr x, mpfr_rnd_t rnd)
       mpfr_set_prec (tmp, prec);
       mpfr_set_prec (ump, prec);
     }
-  MPFR_ZIV_FREE (loop);                  /* Free the ZivLoop Controler */
+  MPFR_ZIV_FREE (loop);                  /* Free the ZivLoop Controller */
 
   inex = mpfr_set (y, tmp, rnd);    /* Set y to the computed value */
   mpfr_clear (tmp);
